VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsModules"
Attribute VB_GlobalNameSpace = True
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'---------------------------------------------------------------------------------------
' Module    : clsModules
' DateTime  : 06/02/2007 13:02
' Author    : Vb System Library
' Copyright : (C) 2007 L'quipe Vb System Library
'              See "modPresentation" for more information
' Purpose   : Grer les modules des processus
'---------------------------------------------------------------------------------------
Option Explicit

Public Type ModuleInfos
    ModuleID            As Long     ' ID du module
    ProcessID           As Long     ' ID du processus qui utilise le module
    lpBaseOfDll         As Long     ' Adresse de base de la dll
    EntryPoint          As Long     ' point d'entree de la dll
    SizeOfImage         As Long     ' taille du module en mmoire
    hModule             As Long     ' HMODULE du module
    ModuleName          As String   ' nom du module
    ExeFile             As String   ' path du module
End Type

Private Files As clsFiles   'Used to retrieve files module informations
'---------------------------------------------------------------------------------------
' Procedure : CreateModuleList
' DateTime  : 06/02/2007 13:13
' Author    : Vb System Library
' Purpose   : Cr une liste des modules d'un processus
' Params    : PID       : ID du processus
'             mdList()  : Tableau des ID des modules d'un processus
' Return    : Modules Count
'---------------------------------------------------------------------------------------

Public Function CreateModuleList(ByVal ProcessID As Long, ByRef ModList() As ModuleInfos) As Long
        
    Dim ModName     As String
    Dim ModCount    As Long
    Dim hProcess    As Long
    Dim hMods()     As Long
    Dim cbNeeded    As Long
    Dim i           As Long
    Dim Namelen     As Long
    Dim ModInfo     As MODULEINFO
    Dim ModInfos    As ModuleInfos
    Dim sTmp        As String
        
    hProcess = OpenProcess(PROCESS_QUERY_INFORMATION Or PROCESS_VM_READ, 0, ProcessID)
    If hProcess Then
        ReDim hMods(1024)
        If EnumProcessModules(hProcess, hMods(0), 4096, cbNeeded) Then
            ModCount = ((cbNeeded / 4) - 1)
            ReDim ModList(ModCount) As ModuleInfos
            For i = 0 To ModCount
                GetModuleInformation hProcess, hMods(i), ModInfo, Len(ModInfo)
                ModName = String$(1024, 0)
                Namelen = GetModuleFileNameExA(hProcess, hMods(i), ModName, Len(ModName))
                ModName = Left$(ModName, Namelen)
                With ModList(i)
                    .ModuleID = i
                    .hModule = hMods(i)
                    .ProcessID = ProcessID
                    .lpBaseOfDll = ModInfo.lpBaseOfDll
                    .SizeOfImage = ModInfo.SizeOfImage
                    .EntryPoint = ModInfo.EntryPoint
                    sTmp = Left$(ModName, 2)
                    If sTmp = "\?" Then
                        .ExeFile = Replace(ModName, "\??\", "")
                    ElseIf sTmp = "\S" Then
                        .ExeFile = Replace(ModName, "\SystemRoot", Files.GetWindowsPath)
                    Else
                        .ExeFile = ModName
                    End If
                    .ModuleName = Files.GetFileNameFromFullPath(ModName)
                End With
            Next i
        End If
        CloseHandle hProcess
    End If
    
    Erase hMods
    CreateModuleList = ModCount
    
End Function


'---------------------------------------------------------------------------------------
' Procedure : LoadModuleInProcess
' DateTime  : 06/02/2007 13:02
' Author    : Vb System Library
' Purpose   : Charge un module dans un processus (injecter)
' Params    : ProcessId : ID du processus vis
'               ModulePathName : Chemin d'accs au module (fichier DLL)
'---------------------------------------------------------------------------------------
Public Function LoadModuleInProcess(ByVal ProcessID As Long, ByVal ModulePathName As String) As Long
        
    Dim hKernel32   As Long
    Dim hThread     As Long
    Dim hVirtual    As Long
    Dim hProcess    As Long
    Dim hFunc       As Long
    
    'Ouverture du process
    hProcess = OpenProcess(PROCESS_CREATE_THREAD Or PROCESS_VM_OPERATION Or PROCESS_VM_WRITE Or PROCESS_VM_READ, False, ProcessID)
    
    If hProcess Then
        'On recupere le Handle de Kernel32 qui contient la fontion LoadLibrary
        hKernel32 = GetModuleHandleA(KRN_DLL)
        
        'On recupere l'adresse de la fonction LoadLibrary
        hFunc = GetProcAddress(hKernel32, KRN_LOAD)
        
        'On alloue une zone memoire dans le process de destination
        hVirtual = VirtualAllocEx(hProcess, ByVal 0&, Len(ModulePathName), MEM_COMMIT, PAGE_READWRITE)
        
        'On ecrit le chemin du module dans la zone memoire que l'on vient d'allouer
        If WriteProcessMemory(hProcess, ByVal hVirtual, ByVal ModulePathName, Len(ModulePathName), ByVal 0&) Then
            
            'Creation d'un Thread dans le Process,
            'ce Thread va faire appel a LoadLibrary avec pour parametre hVirtual
            'qui pointe vers la zone contenant le nom du module a charger
            hThread = CreateRemoteThread(hProcess, ByVal 0&, 0, ByVal hFunc, ByVal hVirtual, 0, ByVal 0&)
            If hThread Then
                'Maitenant on attend la fin du Thread
                'Ici on attend 2,5 secondes ce qui est plus que suffisant pour
                'charger un module
                WaitForSingleObject hThread, &H9C4
                'On recupere code de sortie du Thread qui est le code de
                'retour de la fonction LoadLibrary et qui contient le Handle
                'du module charg si tout s'est bien pass
                GetExitCodeThread hThread, LoadModuleInProcess
                'On ferme le Handle du Thread
                CloseHandle hThread
            End If
            
        End If
        
        'On libere la zone alloue dans le Process
        VirtualFreeEx hProcess, hVirtual, Len(ModulePathName), MEM_RELEASE
          
        'On ferme le Handle du Process
        CloseHandle hProcess
    End If
    
End Function


'---------------------------------------------------------------------------------------
' Procedure : UnLoadModuleFromProcess
' DateTime  : 06/02/2007 13:04
' Author    : Vb System Library
' Purpose   : Dcharge un module du processus vis
' Params    : ProcessId : ID du processus vis
'               Pour dcharger le module, on peut donner
'               - Le chemin d'accs  la DLL : ModulePathName
'               - Le handle du module charg en mmoire
'---------------------------------------------------------------------------------------
Public Function UnLoadModuleFromProcess(ByVal ProcessID As Long, Optional ByVal ModulePathName As String = "", Optional ByVal hMod As Long = 0) As Long
    
    Dim hKernel32   As Long
    Dim hThread     As Long
    Dim hProcess    As Long
    Dim hFunc       As Long
    Dim hVirtual    As Long
    
    hProcess = OpenProcess(PROCESS_CREATE_THREAD Or PROCESS_VM_OPERATION Or PROCESS_VM_WRITE Or PROCESS_VM_READ, False, ProcessID)
  
    If hProcess Then
        hKernel32 = GetModuleHandleA(KRN_DLL)
        hFunc = GetProcAddress(hKernel32, GET_HMOD)
        
        ' Si on a pas le handle du module on le recupere en faisant appel
        ' a GetModuleHandle depuis le process cible
        If Not hMod Then
            hVirtual = VirtualAllocEx(hProcess, ByVal 0&, Len(ModulePathName), MEM_COMMIT, PAGE_READWRITE)
            If WriteProcessMemory(hProcess, ByVal hVirtual, ByVal ModulePathName, Len(ModulePathName), ByVal 0&) Then
                hThread = CreateRemoteThread(hProcess, ByVal 0&, 0, ByVal hFunc, ByVal hVirtual, 0, ByVal 0&)
                If hThread Then
                    WaitForSingleObject hThread, &H9C4
                    'on recupere le handle du Module dans hMod
                    GetExitCodeThread hThread, hMod
                    CloseHandle hThread
                End If
            End If
        End If
        
        'Si on le handle
        If hMod Then
            hFunc = GetProcAddress(hKernel32, KRN_FREE)
            hThread = CreateRemoteThread(hProcess, ByVal 0&, 0, ByVal hFunc, ByVal hMod, 0, ByVal 0&)
            If hThread Then
                WaitForSingleObject hThread, &H9C4
                'on recupere le code de retour de FreeLibrary (si = 1 le module a bien t decharg)
                GetExitCodeThread hThread, UnLoadModuleFromProcess
                CloseHandle hThread
            End If
        End If
        
        CloseHandle hProcess
    End If
    
End Function

Private Sub Class_Initialize()
    Set Files = New clsFiles
End Sub

Private Sub Class_Terminate()
    Set Files = Nothing
End Sub
